//
//  DBHandle.m
//  Freem
//
//  Created by mulp on 04/03/13.
//  author: Fabio Cuomo : me@devme.it
//

#import "DBHandle.h"

@interface DBHandle () {
    
}

/**
 * Check if the SQL database has already been saved to the users phone,
 * if not then copy it over.
 */
-(void) checkAndCreateDatabase;

@end

// Usage: [[DBHandle getInstance] method_name];
@implementation DBHandle

@synthesize databasePath;

static DBHandle *instance = nil;
NSString * const databaseName = @"freem.db";

+(DBHandle*) sharedInstance {
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        instance = [[self alloc] init];
    });
    return instance;
}

-(id) init {
	self = [super init];
	if (self != nil) {
		[self checkAndCreateDatabase];
        NSLog(@"dbhandle creato!");
	}
	return self;
}

-(int) connectDB {
    // Open the database.
	int status = sqlite3_open([databaseName UTF8String], &dbconn);
    if (status != SQLITE_OK) {
        // Even though the open failed, call close to properly clean up resources.
        sqlite3_close(dbconn);
        NSAssert1(0, @"Failed to open database with message '%s'.", sqlite3_errmsg(dbconn));
        // Additional error handling, as appropriate...
	}
	return status;
}

-(void) disconnectDB {
	sqlite3_close(dbconn);
}

-(void) checkAndCreateDatabase{
	
	// Get the path to the documents directory and append the databaseName
	NSArray *homePaths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *homeDir = [homePaths objectAtIndex:0];
	self.databasePath = [homeDir stringByAppendingPathComponent:databaseName];
	NSLog(@"Database Path: %@", databasePath);
	
	// Get the path to the database in the application package
	NSString *databasePathFromApp = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:databaseName];
	NSLog(@"DBPATH: %@", databasePathFromApp);
    
	BOOL success;
	
	// Create a FileManager object, we will use this to check the status
	// of the database and to copy it over if required
	NSFileManager *fileManager = [NSFileManager defaultManager];
	
	// Check if the database has already been created in the users filesystem
	success = [fileManager fileExistsAtPath:databasePath];
	
	NSError *err;
	// If the database already exists then return without doing anything
	if (success) {
		//[fileManager removeItemAtPath:databasePath error:&err];
		return;
	}
	
	// Copy the database from the package to the users filesystem
	success = [fileManager copyItemAtPath:databasePathFromApp toPath:databasePath error:&err];
	NSLog(@"Src = %@", databasePathFromApp);
	NSLog(@"Dst = %@", databasePath);
	NSLog(@"Copy status = %d -- %@", success, [err description]);
}

- (BOOL)isClientRegistered {
    return ![[self getSetting:@"PRIVATE_KEY"] isEqualToString:@""];
}

- (BOOL)setSetting:(NSString *)pref value:(NSString *)value {
	BOOL result = TRUE;
	// Open the database from the users filessytem
    NSLog(@"open setSetting");
	if(sqlite3_open([self.databasePath UTF8String], &dbconn) == SQLITE_OK) {
		
		// Setup the SQL Statement and compile it for faster access
        const char *sqlStatement = "DELETE FROM settings WHERE pref=?";
		sqlite3_stmt *compiledStatement;
        
        NSLog(@"prepare");
		if(sqlite3_prepare_v2(dbconn, sqlStatement, -1, &compiledStatement, NULL) != SQLITE_OK) {
			NSLog(@"Error: failed to prepare stmt with message %s", sqlite3_errmsg(dbconn));
			result = FALSE;
		}
        
		sqlite3_bind_text(compiledStatement, 1, [pref UTF8String], -1, SQLITE_TRANSIENT);
		if (sqlite3_step(compiledStatement) != SQLITE_DONE) {
			NSAssert(1, @"Error: failed to remove pref from setting %s", sqlite3_errmsg(dbconn));
		}
        NSLog(@"finalize");
		sqlite3_finalize(compiledStatement);
        
        NSLog(@"prepare");
        sqlStatement = "INSERT INTO settings (pref, value) VALUES (?,?) ";
		if(sqlite3_prepare_v2(dbconn, sqlStatement, -1, &compiledStatement, NULL) != SQLITE_OK) {
			NSLog(@"Error: failed to prepare stmt with message %s", sqlite3_errmsg(dbconn));
			result = FALSE;
		}

		sqlite3_bind_text(compiledStatement, 1, [pref UTF8String], -1, SQLITE_TRANSIENT);
		sqlite3_bind_text(compiledStatement, 2, [value UTF8String], -1, SQLITE_TRANSIENT);
		if (sqlite3_step(compiledStatement) != SQLITE_DONE) {
			NSAssert(1, @"Error: failed to insert prefs in setting %s", sqlite3_errmsg(dbconn));
		}
        
        NSLog(@"finalize");
        // Release the compiled statement from memory
		sqlite3_finalize(compiledStatement);
    }
    NSLog(@"close");
    sqlite3_close(dbconn);
    return result;
    
}

- (NSString *)getSetting:(NSString *)pref {
    NSString *setting = @"";
	// Open the database from the users filessytem
    NSLog(@"open getSetting");
	if(sqlite3_open([self.databasePath UTF8String], &dbconn) == SQLITE_OK) {
		// Setup the SQL Statement and compile it for faster access
		NSString * sql = @"SELECT value FROM settings WHERE pref=? ";
		
		const char *sqlStatement = [sql UTF8String];
		sqlite3_stmt *compiledStatement;
		
        NSLog(@"prepare");
		if(sqlite3_prepare_v2(dbconn, sqlStatement, -1, &compiledStatement, NULL) != SQLITE_OK) {
			NSAssert(1, @"Error: failed to prepare stmt with message %s", sqlite3_errmsg(dbconn));
		}
		sqlite3_bind_text(compiledStatement, 1, [pref UTF8String], -1, SQLITE_TRANSIENT);
        
		if (sqlite3_step(compiledStatement) == SQLITE_ROW) {
            setting = [NSString stringWithUTF8String:(char *)sqlite3_column_text(compiledStatement, 0)];
		}
        
		// Release the compiled statement from memory
        NSLog(@"finalize");
		sqlite3_finalize(compiledStatement);
        NSLog(@"close");
		sqlite3_close(dbconn);
	}
    return setting;
}

- (NSMutableArray *)getThreads {
	NSMutableArray *threads = [[NSMutableArray alloc] init];
	
	NSLog(@"Path is: %@", databasePath);
	// Open the database from the users filessytem
    NSLog(@"open getThreads");
	if(sqlite3_open([self.databasePath UTF8String], &dbconn) == SQLITE_OK) {
		const char *sqlStatement = "SELECT id, phone, unread, last_update FROM threads ORDER BY last_update DESC";
		sqlite3_stmt *compiledStatement;
		
        NSLog(@"prepare");
		if(sqlite3_prepare_v2(dbconn, sqlStatement, -1, &compiledStatement, NULL) != SQLITE_OK) {
			NSLog(@"Error: failed to prepare stmt with message %s", sqlite3_errmsg(dbconn));
		}
		Thread *th;
		while (sqlite3_step(compiledStatement) == SQLITE_ROW) {
			th = [Thread alloc];
			th.idthread = [NSNumber numberWithInt:sqlite3_column_int(compiledStatement, 0)];
			th.phone = [NSString stringWithUTF8String:(char *)sqlite3_column_text(compiledStatement, 1)];
			th.unread = [NSNumber numberWithInt:sqlite3_column_int(compiledStatement, 2)];
			th.lastUpdate = [NSNumber numberWithInt:sqlite3_column_int(compiledStatement, 3)];
			[threads addObject:th];
		}
		
		// Release the compiled statement from memory
        NSLog(@"finalize");
		sqlite3_finalize(compiledStatement);
        NSLog(@"close");
		sqlite3_close(dbconn);
	}
	return threads;    
}

-(NSMutableArray*) retrieveConversation:(int)idthread {
	NSMutableArray *msg = [[NSMutableArray alloc] init];
	
	// Open the database from the users filessytem
    NSLog(@"open retrieveConversation");
	if(sqlite3_open([self.databasePath UTF8String], &dbconn) == SQLITE_OK) {
		// Setup the SQL Statement and compile it for faster access
		NSString * sql = @"SELECT message, direction, ts_send FROM messages WHERE thread=? ORDER BY id";
		
		const char *sqlStatement = [sql UTF8String];
		sqlite3_stmt *compiledStatement;
		
        NSLog(@"prepare");
		if(sqlite3_prepare_v2(dbconn, sqlStatement, -1, &compiledStatement, NULL) != SQLITE_OK) {
			NSLog(@"Error: failed to prepare stmt with message %s", sqlite3_errmsg(dbconn));
		}
        sqlite3_bind_int(compiledStatement, 1, idthread);
        
        Message *message;
		while(sqlite3_step(compiledStatement) == SQLITE_ROW) {
            message = [Message alloc];
            message.idthread = [NSNumber numberWithInt:idthread];
            message.message = [NSString stringWithUTF8String:(char *)sqlite3_column_text(compiledStatement, 0)];
            message.direction = [NSNumber numberWithInt:sqlite3_column_int(compiledStatement, 1)];
            message.tsSend = [NSNumber numberWithInt:sqlite3_column_int(compiledStatement, 2)];
			[msg addObject: message];
		}
		
		// Release the compiled statement from memory
        NSLog(@"finalize");
		sqlite3_finalize(compiledStatement);
        
        sqlStatement = "UPDATE threads SET unread=? WHERE id = ? ";
        NSLog(@"prepare");
		if(sqlite3_prepare_v2(dbconn, sqlStatement, -1, &compiledStatement, NULL) != SQLITE_OK) {
			NSAssert1(0, @"Error: failed to prepare stmt with message %s", sqlite3_errmsg(dbconn));
		}
        
        sqlite3_bind_int(compiledStatement,  1, 0);
        sqlite3_bind_int(compiledStatement,  2, idthread);
        
		if (sqlite3_step(compiledStatement) != SQLITE_DONE) {
			NSAssert1(0, @"Error while updating current thread %s", sqlite3_errmsg(dbconn));
		}
		// Release the compiled statement from memory
        NSLog(@"finalize");
		sqlite3_finalize(compiledStatement);
        
        
        
        NSLog(@"close");
		sqlite3_close(dbconn);
	}
	return msg;
}

- (BOOL)registerClient:(NSString *)privateKey {
    return [self setSetting:@"PRIVATE_KEY" value:privateKey];
}

- (int)registerLocalMessage:(NSString *)msg idthread:(int)idthread {
	int idmsg = 0;
	// Open the database from the users filessytem
    NSLog(@"open registerLocalMessage");
	if(sqlite3_open([self.databasePath UTF8String], &dbconn) == SQLITE_OK) {
		
		// Setup the SQL Statement and compile it for faster access
        const char *sqlStatement = "INSERT INTO messages (thread, message, direction, ts_send) VALUES (?,?,?,?)";
		sqlite3_stmt *compiledStatement;

        NSLog(@"prepare");
		if(sqlite3_prepare_v2(dbconn, sqlStatement, -1, &compiledStatement, NULL) != SQLITE_OK) {
			NSAssert1(0, @"Error: failed to prepare stmt with message %s", sqlite3_errmsg(dbconn));
		}

        NSTimeInterval timeStamp = [[NSDate date] timeIntervalSince1970];
        sqlite3_bind_int(compiledStatement,  1, idthread);
		sqlite3_bind_text(compiledStatement, 2, [msg UTF8String], -1, SQLITE_TRANSIENT);
        sqlite3_bind_int(compiledStatement,  3, OUTCOMING_MSG);
        sqlite3_bind_int(compiledStatement,  4, [[NSNumber numberWithDouble:timeStamp] intValue]);
		
		if (sqlite3_step(compiledStatement) != SQLITE_DONE) {
            NSAssert1(0, @"Error while storing local message. Msg: %s", sqlite3_errmsg(dbconn));
		}
        
        idmsg = sqlite3_last_insert_rowid(dbconn);
        
		// Release the compiled statement from memory
        NSLog(@"finalize");
		sqlite3_finalize(compiledStatement);
        NSLog(@"close");
        sqlite3_close(dbconn);
    }
    return idmsg;
}

- (BOOL)registerUnreadMessage:(NSString *)msg idthread:(int)idthread tstamp:(int)tstamp {
	BOOL result = TRUE;
	// Open the database from the users filessytem
    NSLog(@"open registerUnreadMessage");
	if(sqlite3_open([self.databasePath UTF8String], &dbconn) == SQLITE_OK) {
		
		// Setup the SQL Statement and compile it for faster access
        const char *sqlStatement = "INSERT INTO messages (thread, message, direction, ts_send) VALUES (?,?,?,?)";
		sqlite3_stmt *compiledStatement;
        
        NSLog(@"prepare");
		if(sqlite3_prepare_v2(dbconn, sqlStatement, -1, &compiledStatement, NULL) != SQLITE_OK) {
			NSLog(@"Error: failed to prepare stmt with message %s", sqlite3_errmsg(dbconn));
			result = FALSE;
		}
        
        sqlite3_bind_int(compiledStatement,  1, idthread);
		sqlite3_bind_text(compiledStatement, 2, [msg UTF8String], -1, SQLITE_TRANSIENT);
        sqlite3_bind_int(compiledStatement,  3, INCOMING_MSG);
        sqlite3_bind_int(compiledStatement,  4, tstamp);
		
		if (sqlite3_step(compiledStatement) != SQLITE_DONE) {
			NSAssert1(0, @"Error: failed to prepare stmt with message %s", sqlite3_errmsg(dbconn));
			result = FALSE;
		}
		// Release the compiled statement from memory
        NSLog(@"finalize");
		sqlite3_finalize(compiledStatement);
        NSLog(@"close");
        sqlite3_close(dbconn);
    }
    return result;
}

- (int)getThreadIdByPhone:(NSString *)phone {
    int idthread = 0;
	// Open the database from the users filessytem
    NSLog(@"open getThreadIdByPhone");
	if(sqlite3_open([self.databasePath UTF8String], &dbconn) == SQLITE_OK) {
		// Setup the SQL Statement and compile it for faster access
		
		const char *sqlStatement = "SELECT id FROM threads WHERE phone=? ";
		sqlite3_stmt *compiledStatement;
		
        NSLog(@"prepare");
		if(sqlite3_prepare_v2(dbconn, sqlStatement, -1, &compiledStatement, NULL) != SQLITE_OK) {
			NSAssert1(0, @"Error: failed to prepare stmt with message %s", sqlite3_errmsg(dbconn));
		}
		sqlite3_bind_text(compiledStatement, 1, [phone UTF8String], -1, SQLITE_TRANSIENT);
        
        // Il thread non esiste sul phone number corrente
        if (sqlite3_step(compiledStatement) == SQLITE_ROW) {
            idthread = sqlite3_column_int(compiledStatement, 0);
        }
		// Release the compiled statement from memory
        NSLog(@"finalize");
		sqlite3_finalize(compiledStatement);
        NSLog(@"close");
		sqlite3_close(dbconn);
	}
    return idthread;
}

- (int)startNewThread:(NSString *)phone {
    int idthread = 0;
	// Open the database from the users filessytem
    NSLog(@"open startNewThread");
	if(sqlite3_open([self.databasePath UTF8String], &dbconn) == SQLITE_OK) {
		// Setup the SQL Statement and compile it for faster access
		
		const char *sqlStatement = "INSERT INTO threads (phone, unread, last_update) VALUES (?,?,?)";
		sqlite3_stmt *compiledStatement;
		
        NSLog(@"prepare");
		if(sqlite3_prepare_v2(dbconn, sqlStatement, -1, &compiledStatement, NULL) != SQLITE_OK) {
			NSAssert1(0, @"Error: failed to prepare stmt with message %s", sqlite3_errmsg(dbconn));
		}
		
        NSTimeInterval timeStamp = [[NSDate date] timeIntervalSince1970];
        sqlite3_bind_text(compiledStatement, 1, [phone UTF8String], -1, SQLITE_TRANSIENT);
        sqlite3_bind_int(compiledStatement,  2, 1);
        sqlite3_bind_int(compiledStatement,  3, [[NSNumber numberWithDouble:timeStamp] intValue]);
        
        if (sqlite3_step(compiledStatement) != SQLITE_DONE) {
            NSAssert1(0, @"Error while inserting data. '%s'", sqlite3_errmsg(dbconn));
        }
        idthread = sqlite3_last_insert_rowid(dbconn);
        
		// Release the compiled statement from memory
        NSLog(@"finalize");
		sqlite3_finalize(compiledStatement);
        NSLog(@"close");
		sqlite3_close(dbconn);
	}
    return idthread;
}

- (BOOL)updateThread:(NSString *)phone {
    BOOL result = YES;
	// Open the database from the users filessytem
    NSLog(@"open updateThread");
	if(sqlite3_open([self.databasePath UTF8String], &dbconn) == SQLITE_OK) {
		// Setup the SQL Statement and compile it for faster access
		
		const char *sqlStatement = "UPDATE threads SET last_update = ?, unread = unread + 1 WHERE phone=? ";
		sqlite3_stmt *compiledStatement;
		
        NSLog(@"prepare");
		if(sqlite3_prepare_v2(dbconn, sqlStatement, -1, &compiledStatement, NULL) != SQLITE_OK) {
			NSAssert1(0, @"Error: failed to prepare stmt with message %s", sqlite3_errmsg(dbconn));
		}

        NSTimeInterval timeStamp = [[NSDate date] timeIntervalSince1970];
        sqlite3_bind_int(compiledStatement,  1, [[NSNumber numberWithDouble:timeStamp] intValue]);
        sqlite3_bind_text(compiledStatement, 2, [phone UTF8String], -1, SQLITE_TRANSIENT);
        
        if (sqlite3_step(compiledStatement) != SQLITE_DONE) {
            NSAssert1(0, @"Error while updating data. '%s'", sqlite3_errmsg(dbconn));
        }
		
		// Release the compiled statement from memory
        NSLog(@"finalize");
		sqlite3_finalize(compiledStatement);
        NSLog(@"close");
		sqlite3_close(dbconn);
	}
    return result;
}

@end
