//
//  BubbleTableViewCell.m
//  Freem
//
//  Created by mulp on 05/03/13.
//  author: Fabio Cuomo : me@devme.it
//

#import "BubbleTableViewCell.h"

@interface BubbleTableViewCell () {
    BubbleView *bubbleView;
    UILabel *labelCell;
    UIImageView *read;
}
@end

static UIColor *color;

@implementation BubbleTableViewCell

+ (void)initialize {
	if (self == [BubbleTableViewCell class]) {
		color = [UIColor colorWithRed:255/255.0 green:255/255.0 blue:255/255.0 alpha:1.0];
	}
}

- (id)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString*)reuseIdentifier {
	if ((self = [super initWithStyle:style reuseIdentifier:reuseIdentifier])) {
		self.selectionStyle = UITableViewCellSelectionStyleNone;
        
		// Create the speech bubble view
		bubbleView = [[BubbleView alloc] initWithFrame:CGRectZero];
		bubbleView.backgroundColor = color;
		bubbleView.opaque = YES;
		bubbleView.clearsContextBeforeDrawing = NO;
		bubbleView.contentMode = UIViewContentModeRedraw;
		bubbleView.autoresizingMask = 0;
		[self.contentView addSubview:bubbleView];
        
		// Create the label
		labelCell = [[UILabel alloc] initWithFrame:CGRectZero];
		labelCell.backgroundColor = color;
		labelCell.opaque = YES;
		labelCell.clearsContextBeforeDrawing = NO;
		labelCell.contentMode = UIViewContentModeRedraw;
		labelCell.autoresizingMask = 0;
		labelCell.font = [UIFont systemFontOfSize:13];
		labelCell.textColor = [UIColor colorWithRed:64/255.0 green:64/255.0 blue:64/255.0 alpha:1.0];
		[self.contentView addSubview:labelCell];
        
        read = [[UIImageView alloc] initWithFrame:CGRectZero];
        read.image = [UIImage imageNamed:@"read.png"];
        read.hidden = YES;
        [self.contentView addSubview:read];
	}
	return self;
}

- (void)layoutSubviews {
	// This is a little trick to set the background color of a table view cell.
	[super layoutSubviews];
	self.backgroundColor = color;
}

- (void)setMessage:(Message*)message {
	CGPoint point = CGPointZero;
    
    // I messaggi inviati verranno visualizzati all'interno del balloon destro.
    // Quelli ricevuti all'interno del balloon sinistro.
	BubbleType bubbleType;
	if ([message wasSend]) {
		bubbleType = BubbleTypeSend;
		point.x = self.bounds.size.width - message.bubbleSize.width;
		labelCell.textAlignment = NSTextAlignmentRight;
	} else {
		bubbleType = BubbleTypeReceive;
		labelCell.textAlignment = NSTextAlignmentLeft;
	}
    
    // Ridimensiona la view del balloon e visualizza il messaggio
	CGRect rect;
	rect.origin = point;
	rect.size = message.bubbleSize;
	bubbleView.frame = rect;
	[bubbleView setText:message.message bubbleType:bubbleType];
    
	// Formatta la data del messaggio
    NSDate *date = [NSDate dateWithTimeIntervalSince1970:[message.tsSend intValue]];
	NSDateFormatter* formatter = [[NSDateFormatter alloc] init];
	[formatter setDateFormat:@"dd/MM/yyyy HH:mm"];
	NSString* dateString = [formatter stringFromDate:date];
    
	labelCell.text = [NSString stringWithFormat:@"%@", dateString];
	[labelCell sizeToFit];
	labelCell.frame = CGRectMake(8, message.bubbleSize.height, self.contentView.bounds.size.width - 16, 16);
//    if (bubbleType==BubbleTypeReceive && [message.read intValue]==READ_MSG) {
//        read.frame = CGRectMake(120, message.bubbleSize.height - 2, 16, 16);
//        read.hidden = NO;
//    }
//    if (bubbleType==BubbleTypeSend)
//        read.frame = CGRectMake(183, message.bubbleSize.height - 1, 16, 16);
//    else
//        read.frame = CGRectMake(120, message.bubbleSize.height - 2, 16, 16);
//    if ([message.read intValue]==READ_MSG) read.hidden = NO;
}

@end
