//
//  BubbleView.m
//  Freem
//
//  Created by mulp on 08/03/13.
//  author: Fabio Cuomo : me@devme.it
//

#import "BubbleView.h"

const CGFloat VERTICAL_PADDING      = 4;
const CGFloat HORIZONTAL_PADDING    = 4;

const CGFloat TEXT_MARGIN_LEFT      = 17;
const CGFloat TEXT_MARGIN_RIGHT     = 15;
const CGFloat TEXT_MARGIN_TOP       = 10;
const CGFloat TEXT_MARGIN_BOTTOM    = 11;

const CGFloat BUBBLE_MIN_WIDTH  = 50;
const CGFloat BUBBLE_MIN_HEIGHT = 40;

const CGFloat WIDTH_TO_WRAP = 200;

@interface BubbleView () {
    NSString* text;
	BubbleType bubbleType;
    
    UIImage *bubbleSend;
    UIImage *bubbleReceive;
}
@end

@implementation BubbleView

- (id)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
		bubbleSend = [[UIImage imageNamed:@"green.png"] resizableImageWithCapInsets: UIEdgeInsetsMake(15, 24, 16, 25)];
		bubbleReceive = [[UIImage imageNamed:@"grey.png"] resizableImageWithCapInsets: UIEdgeInsetsMake(15, 24, 16, 25)];
    }
    return self;
}


+ (CGSize)sizeForText:(NSString *)text {
	CGSize textSize = [text sizeWithFont:[UIFont systemFontOfSize:[UIFont systemFontSize]]
                       constrainedToSize:CGSizeMake(WIDTH_TO_WRAP, CGFLOAT_MAX)
                           lineBreakMode:NSLineBreakByWordWrapping];
    
	CGSize bubbleSize;
	bubbleSize.width = textSize.width + TEXT_MARGIN_LEFT + TEXT_MARGIN_RIGHT;
	bubbleSize.height = textSize.height + TEXT_MARGIN_TOP + TEXT_MARGIN_BOTTOM;
    
	if (bubbleSize.width < BUBBLE_MIN_WIDTH)
		bubbleSize.width = BUBBLE_MIN_WIDTH;
    
	if (bubbleSize.height < BUBBLE_MIN_HEIGHT)
		bubbleSize.height = BUBBLE_MIN_HEIGHT;
    
	bubbleSize.width += HORIZONTAL_PADDING * 2;
	bubbleSize.height += VERTICAL_PADDING * 2;
    
	return bubbleSize;
}

- (void)setText:(NSString *)newText bubbleType:(BubbleType)newBubbleType {
	text = newText;
	bubbleType = newBubbleType;
	[self setNeedsDisplay];    
}

- (void)drawRect:(CGRect)rect {
	[self.backgroundColor setFill];
	UIRectFill(rect);
    
	CGRect bubbleRect = CGRectInset(self.bounds, VERTICAL_PADDING, HORIZONTAL_PADDING);
    
	CGRect textRect;
	textRect.origin.y = bubbleRect.origin.y + TEXT_MARGIN_TOP;
	textRect.size.width = bubbleRect.size.width - TEXT_MARGIN_LEFT - TEXT_MARGIN_RIGHT;
	textRect.size.height = bubbleRect.size.height - TEXT_MARGIN_TOP - TEXT_MARGIN_BOTTOM;
    
	if (bubbleType == BubbleTypeReceive) {
		[bubbleReceive drawInRect:bubbleRect];
		textRect.origin.x = bubbleRect.origin.x + TEXT_MARGIN_LEFT;
	} else {
		[bubbleSend drawInRect:bubbleRect];
		textRect.origin.x = bubbleRect.origin.x + TEXT_MARGIN_RIGHT;
	}
    
	[[UIColor blackColor] set];
	[text drawInRect:textRect withFont:[UIFont systemFontOfSize:[UIFont systemFontSize]] lineBreakMode:NSLineBreakByWordWrapping];

}

@end
