//
//  FreeMAppDelegate.m
//  Freem
//
//  Created by mulp on 04/03/13.
//  author: Fabio Cuomo : me@devme.it
//

#import "FreeMAppDelegate.h"

void showErrorAlert(NSString* text) {
	UIAlertView* alertView = [[UIAlertView alloc]
                              initWithTitle:text
                              message:nil
                              delegate:nil
                              cancelButtonTitle:NSLocalizedString(@"OK", nil)
                              otherButtonTitles:nil];
    
	[alertView show];
}

@implementation FreeMAppDelegate

@synthesize navController;
@synthesize window, threadViewController;

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {

    self.window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
    
    self.threadViewController = [ThreadsTBViewController alloc];
    Settings *settings = [Settings sharedInstance];
    if (![settings isClientRegistered]) {
        LoginViewController *loginViewController = [[LoginViewController alloc] initWithNibName:@"LoginView" bundle:nil];
        loginViewController.threadViewController = self.threadViewController;
        self.navController = [[UINavigationController alloc] initWithRootViewController:loginViewController];
    } else {
        self.navController = [[UINavigationController alloc] initWithRootViewController:self.threadViewController];
    }
    [self.navController.navigationBar setTintColor:[UIColor lightGrayColor]];
    self.window.rootViewController = self.navController;
    [self.window makeKeyAndVisible];

    // Registra l'app per la ricezione delle notifiche push
    [[UIApplication sharedApplication]
     registerForRemoteNotificationTypes:(UIRemoteNotificationTypeAlert | UIRemoteNotificationTypeBadge | UIRemoteNotificationTypeSound)];
    
    // Verifica se l'app è stata lanciata dall'utente direttamente dalla push
	if (launchOptions != nil) {
		NSDictionary* dictionary = [launchOptions objectForKey:UIApplicationLaunchOptionsRemoteNotificationKey];
		if (dictionary != nil) {
			NSLog(@"Launched from push notification: %@", dictionary);
            [self addReceivedMessage:dictionary];
            [self clearNotification];
		}
	}    

    // Override point for customization after application launch.
    return YES;
}
							
- (void)applicationWillResignActive:(UIApplication *)application {
    // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
    // Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
}

- (void)applicationDidEnterBackground:(UIApplication *)application {
    // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later. 
    // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
}

- (void)applicationWillEnterForeground:(UIApplication *)application {
}

- (void)applicationDidBecomeActive:(UIApplication *)application {
    // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
}

- (void)applicationWillTerminate:(UIApplication *)application {
    // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
}

#pragma mark -
#pragma mark Push Notification

- (void)application:(UIApplication*)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData*)deviceToken {
    // Questo metodo viene richiamato tipicamente quando il device si registra nel sistem
    // per ricevere notifiche di tipo Push. Riceve in input dai sistemi Apple un token
    // che autentica e identifica il device. Il token viene memorizzato sul server
    // e viene verificato se il suo valore è cambiato nel tempo. Infatti ad ogni nuova
    // registrazione il token potrebbe cambiare e di conseguenza il suo valore deve
    // essere aggiornato.
	
    Settings *settings = [Settings sharedInstance];
    NSString *currentToken = settings.tokenKey;
    
	NSString *incomingToken = [deviceToken description];
	incomingToken = [incomingToken stringByTrimmingCharactersInSet:[NSCharacterSet characterSetWithCharactersInString:@"<>"]];
	incomingToken = [incomingToken stringByReplacingOccurrencesOfString:@" " withString:@""];
    
	NSLog(@"My token is: %@", currentToken);
	NSLog(@"Incoming token is: %@", incomingToken);
    
    // Se il token è cambiato e l'utente è già registrato su Free Message allora il
    // valore del token viene aggiornato.
	if ([settings isClientRegistered] && ![incomingToken isEqualToString:currentToken]) {
        [settings updateTokenKey:incomingToken];
	} else {
        settings.tokenKey = incomingToken;
    }
}

- (void)application:(UIApplication*)application didFailToRegisterForRemoteNotificationsWithError:(NSError*)error {
	NSLog(@"Failed to get token, error: %@", error);
}

- (void)application:(UIApplication*)application didReceiveRemoteNotification:(NSDictionary*)userInfo {
    // Questo metodo viene invocato quando l'app riceve una Push Notification
	NSLog(@"Received notification: %@", userInfo);
    
    [self addReceivedMessage:userInfo];
    [self clearNotification];
}

-(void)addReceivedMessage:(NSDictionary*)userInfo {
    NSDictionary *aps = [userInfo objectForKey:@"aps"];

    ServerCalls *serverCall = [ServerCalls sharedInstance];
    NSString *type = [aps objectForKey:@"type"];    
    if (type!=nil) {
        NSURLConnection *conn;
        if ([type isEqualToString:@"message"]) {
            dbhandle = [DBHandle sharedInstance];
            RetrieveMessageDelegate *rmd = [[RetrieveMessageDelegate alloc] init];
            rmd.threadViewController = self.threadViewController;
            conn = [serverCall retrieve:rmd];
            
            if (!conn) {
                showErrorAlert(NSLocalizedString(@"Errore durante il recupero del messaggio.", nil));
                return;
            }
        }
    }
}

-(void)clearNotification {
    [[UIApplication sharedApplication] setApplicationIconBadgeNumber:1];
    [[UIApplication sharedApplication] setApplicationIconBadgeNumber:0];
    [[UIApplication sharedApplication] cancelAllLocalNotifications];
}
@end
