//
//  FreeMThreads.m
//  Freem
//
//  Created by mulp on 05/03/13.
//  author: Fabio Cuomo : me@devme.it
//

#import "FreeMThreads.h"

@interface FreeMThreads () {
    NSMutableArray *datasource;
    DBHandle *dbhandle;
    BOOL keyboardIsShown;
    int lastInsertedMessageId;
}

@end

@implementation FreeMThreads

@synthesize idthread, phoneNumber;


- (void)viewDidLoad {
    NSLog(@"viewDidLoad");
    [super viewDidLoad];

    UIBarButtonItem *composeButton = [[UIBarButtonItem alloc]
                                         initWithImage:[UIImage imageNamed:@"compose.png"]
                                         style:UIBarButtonItemStylePlain
                                         target:self action:@selector(messageToNumber)];
    
    self.navigationItem.rightBarButtonItem = composeButton;
    
    self.messages.separatorStyle = UITableViewCellSeparatorStyleNone;
    self.messages.dataSource = self;
    self.messages.delegate = self;
    
    self.navigationItem.title = [Utility retrieveContactNameFromAddressBookByPhone:self.phoneNumber];
    UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
    button.frame = CGRectMake(0,0,32,32);
    [button setBackgroundImage:[UIImage imageNamed:@"freem-icon.png"] forState:UIControlStateNormal];
    
    [button addTarget:self.navigationController action:@selector(popViewControllerAnimated:) forControlEvents:UIControlEventTouchUpInside];
    UIBarButtonItem *barButtonItem = [[UIBarButtonItem alloc] initWithCustomView:button];
    [self.navigationItem setLeftBarButtonItem:barButtonItem];
    
    // register for keyboard notifications
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWillShow:)
                                                 name:UIKeyboardWillShowNotification
                                               object:self.view.window];
    // register for keyboard notifications
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWillHide:)
                                                 name:UIKeyboardWillHideNotification
                                               object:self.view.window];
    keyboardIsShown = NO;
}

- (void)scrollToLastMessage {
    if ([datasource count]>0) {
        NSIndexPath *indexPath = [NSIndexPath indexPathForRow:[datasource count]-1 inSection:0];
        [self.messages scrollToRowAtIndexPath:indexPath
                         atScrollPosition:UITableViewScrollPositionBottom
                                 animated:YES];
    }
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    if ([self isViewLoaded]) {
        [self refreshConversation];
        [self scrollToLastMessage];
    }
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)refreshConversation {
    self.navigationItem.title = [Utility retrieveContactNameFromAddressBookByPhone:self.phoneNumber];
    dbhandle = [DBHandle sharedInstance];
    datasource = [dbhandle retrieveConversation:[self.idthread integerValue]];
    [dbhandle disconnectDB];

    [self.messages reloadData];
}

- (void) messageToNumber {
    MsgToNumberViewController *msgView = [[MsgToNumberViewController alloc] initWithNibName:@"MsgToNumber" bundle:nil];
    msgView.contact = self.phoneNumber;
    [self.navigationController pushViewController:msgView animated:YES];
}

#pragma mark -
#pragma mark Send message method

- (IBAction)sendMessage:(id)sender {
    self.msg.text = [self.msg.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
    if (![self.msg.text isEqualToString:@""]) {
        dbhandle = [DBHandle sharedInstance];
        lastInsertedMessageId = [dbhandle registerLocalMessage:self.msg.text idthread:[self.idthread intValue]];
        Message *msg = [Message alloc];
        msg.idthread = self.idthread;
        msg.message = self.msg.text;
        msg.direction = 0;
        msg.read = [NSNumber numberWithInt:0];
        msg.tsSend = [NSNumber numberWithInt:[[NSDate date] timeIntervalSince1970]];
        msg.messageId = 0;
        [datasource addObject:msg];
        [self.messages reloadData];
        self.msg.text = @"";
        Settings *settings = [Settings sharedInstance];
        ServerCalls *serverCall = [ServerCalls sharedInstance];
        [serverCall send:settings.phone recipient:self.phoneNumber message:msg.message delegate:self];
    }
    [self.msg resignFirstResponder];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
    // Risposta da parte del server. Endpoint recupero messaggio
    NSString* resp = [NSString stringWithUTF8String:[data bytes]];
    NSLog(@"Data: %@", resp);
    
    NSError* error;
    
    // JSON della risposta
    NSDictionary* json = [NSJSONSerialization JSONObjectWithData:data
                                                         options:kNilOptions error:&error];
    NSNumber *code = [json objectForKey:@"code"];
    if ([code intValue]!=0) {
        NSDictionary *result = [json objectForKey:@"result"];
        showErrorAlert([result objectForKey:@"description"]);
    }
}

#pragma mark -
#pragma mark Keyboard methods

- (void)keyboardWillHide:(NSNotification *)notification {
    NSDictionary* userInfo = [notification userInfo];
    
    // get the size of the keyboard
    CGSize keyboardSize = [[userInfo objectForKey:UIKeyboardFrameBeginUserInfoKey] CGRectValue].size;
    
    // resize the scrollview
    CGRect viewFrame = self.view.frame;
    viewFrame.size.height += keyboardSize.height;
    
    CGRect msgViewFrame = self.messages.frame;
    msgViewFrame.size.height += keyboardSize.height - 100;
    
    [UIView beginAnimations:nil context:NULL];
    [UIView setAnimationBeginsFromCurrentState:YES];
    // The kKeyboardAnimationDuration I am using is 0.3
    [UIView setAnimationDuration:0.3];
    [self.view setFrame:viewFrame];
    [UIView commitAnimations];
    
    keyboardIsShown = NO;
    [self scrollToLastMessage];
}

- (void)keyboardWillShow:(NSNotification *)notification {
    if (keyboardIsShown) {
        return;
    }
    
    NSDictionary* userInfo = [notification userInfo];
    
    // get the size of the keyboard
    CGSize keyboardSize = [[userInfo objectForKey:UIKeyboardFrameBeginUserInfoKey] CGRectValue].size;
    
    // resize the noteView
    CGRect viewFrame = self.view.frame;
    viewFrame.size.height -= keyboardSize.height;
    
    CGRect msgViewFrame = self.messages.frame;
    msgViewFrame.size.height -= keyboardSize.height;

    [UIView beginAnimations:nil context:NULL];
    [UIView setAnimationBeginsFromCurrentState:YES];
    // The kKeyboardAnimationDuration I am using is 0.3
    [UIView setAnimationDuration:0.3];
    [self.view setFrame:viewFrame];
    [self.messages setFrame:msgViewFrame];
    [UIView commitAnimations];
    
    keyboardIsShown = YES;
    [self scrollToLastMessage];
}

#pragma mark -
#pragma mark UITableViewDataSource

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [datasource count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *CellIdentifier = @"Cell";
    
    BubbleTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[BubbleTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
        cell.accessoryType = UITableViewCellAccessoryNone;
    }
    
    Message *aMsg = [datasource objectAtIndex:indexPath.row];
    [cell setMessage:aMsg];
    return cell;
}

#pragma mark -
#pragma mark UITableView Delegate

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
	Message* message = [datasource objectAtIndex:indexPath.row];
	message.bubbleSize = [BubbleView sizeForText:message.message];
    NSLog(@"Message: %@  - bsize: %f,%f - btype: %@", message.message, message.bubbleSize.width, (message.bubbleSize.height+16), message.direction);
	return message.bubbleSize.height + 16;
}

@end
