//
//  MsgToNumberViewController.m
//  Freem
//
//  Created by mulp on 13/03/13.
//  author: Fabio Cuomo : me@devme.it
//

#import "MsgToNumberViewController.h"

@interface MsgToNumberViewController () {
    int lastInsertedMessageId;
}
@end

@implementation MsgToNumberViewController

@synthesize phoneNumber;
@synthesize message;
@synthesize contact;
@synthesize idthread;

- (void)viewDidLoad {
    [super viewDidLoad];

    UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
    button.frame = CGRectMake(0,0,32,32);
    [button setBackgroundImage:[UIImage imageNamed:@"freem-icon.png"] forState:UIControlStateNormal];
    
    [button addTarget:self.navigationController action:@selector(popViewControllerAnimated:) forControlEvents:UIControlEventTouchUpInside];
    UIBarButtonItem *barButtonItem = [[UIBarButtonItem alloc] initWithCustomView:button];
    [self.navigationItem setLeftBarButtonItem:barButtonItem];
    
    UIBarButtonItem *sendMessageButton = [[UIBarButtonItem alloc]
                                          initWithImage:[UIImage imageNamed:@"send.png"]
                                          style:UIBarButtonItemStyleBordered
                                          target:self action:@selector(sendMessage)];
    
    self.navigationItem.rightBarButtonItem = sendMessageButton;

    self.message.layer.borderWidth = 1.0f;
    self.message.layer.borderColor = [[UIColor blackColor] CGColor];
    
    if ([self.contact isEqualToString:@""]) {
        self.navigationItem.title = @"Nuovo messaggio";
        [self.phoneNumber becomeFirstResponder];
    } else {
        self.navigationItem.title = [Utility retrieveContactNameFromAddressBookByPhone:contact];
        self.phoneNumber.text = contact;
        [self.message becomeFirstResponder];
    }
}

- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
    [self.navigationController popViewControllerAnimated:NO];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)sendMessage {
    NSError *error;
    NSRegularExpression *regex = [NSRegularExpression
                                  regularExpressionWithPattern: @"^\\+\\d{12}$"
                                  options:0
                                  error:&error];
    NSRange range = [regex rangeOfFirstMatchInString:self.phoneNumber.text
                            options:0
                            range:NSMakeRange(0, [self.phoneNumber.text length])];
    
    if (range.location == NSNotFound) {
        showErrorAlert(@"Formato del numero errato.");
        return;
    }
    self.message.text = [self.message.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
    if ([self.message.text isEqualToString:@""]) {
        showErrorAlert(@"Il campo messaggio non può essere vuoto.");
        return;
    }

    Settings *settings = [Settings sharedInstance];
    ServerCalls *serverCall = [ServerCalls sharedInstance];
    [serverCall send:settings.phone recipient:self.phoneNumber.text message:self.message.text delegate:self];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
    // Risposta da parte del server. Endpoint recupero messaggio
    NSString* resp = [NSString stringWithUTF8String:[data bytes]];
    NSLog(@"send response: %@", resp);
    
    NSError* error;
    
    // JSON della risposta
    NSDictionary* json = [NSJSONSerialization JSONObjectWithData:data
                                                         options:kNilOptions error:&error];
    NSNumber *code = [json objectForKey:@"code"];
    if ([code intValue]==0) {
        DBHandle *dbhandle = [DBHandle sharedInstance];
        idthread = [dbhandle getThreadIdByPhone:self.phoneNumber.text];
        if (idthread==0) {
            idthread = [dbhandle startNewThread:self.phoneNumber.text];
        } else {
            [dbhandle updateThread:self.phoneNumber.text];
        }
        lastInsertedMessageId = [dbhandle registerLocalMessage:self.message.text idthread:idthread];
        
        FreeMAppDelegate *appDelegate = (FreeMAppDelegate*) [UIApplication sharedApplication].delegate;
        [self.navigationController popViewControllerAnimated:NO];
        [appDelegate.threadViewController didSaveMessage:idthread phone:self.phoneNumber.text];
    } else if ([code intValue]==4) {
        showErrorAlert(@"Il destinatario selezionato non è iscritto a FreeMessage.");
    }
}

@end
