//
//  ServerCalls.m
//  Freem
//
//  Created by mulp on 07/03/13.
//  author: Fabio Cuomo : me@devme.it
//

#import "ServerCalls.h"

@interface ServerCalls ()


-(NSURLConnection*) call:(NSString*) serverURL endpoint:(NSString*)endpoint params:(NSMutableDictionary*)params delegate:(id)delegate;
@end

@implementation ServerCalls

static ServerCalls *instance = nil;

+(ServerCalls*) sharedInstance {
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        instance = [[self alloc] init];
    });
    return instance;
}

-(id) init {
	self = [super init];
	if (self != nil) {}
	return self;
}

- (NSURLConnection*)call:(NSString *)serverURL endpoint:(NSString *)endpoint params:(NSMutableDictionary *)params delegate:(id)delegate{
    
    NSString *url = [[serverURL stringByAppendingString:@"/"] stringByAppendingString:endpoint];
    NSMutableURLRequest *request = [[NSMutableURLRequest alloc]
                                    initWithURL: [NSURL URLWithString: url]];
    [request setHTTPMethod:@"POST"];
    [request setTimeoutInterval:10000];
    [request addValue:@"application/x-www-form-urlencoded" forHTTPHeaderField:@"Content-Type"];
    
    NSString *postString = @"";
    int paramCounter = 0;
    for (id key in params) {
        if (paramCounter>0) {
            postString = [postString stringByAppendingString:@"&"];
        }
        postString = [postString stringByAppendingString:(NSString*)key];
        postString = [postString stringByAppendingString:@"="];
        id value = [params objectForKey:key];
        if ([value isKindOfClass:[NSNumber class]]) {
            postString = [postString stringByAppendingString: [((NSNumber*)value) stringValue]];
        } else {
            postString = [postString stringByAppendingString: [((NSString*)value) urlencode]];
        }
        paramCounter++;
    }
    [request setValue:[NSString stringWithFormat:@"%d", [postString length]] forHTTPHeaderField:@"Content-length"];
    [request setHTTPBody:[postString dataUsingEncoding:NSUTF8StringEncoding]];
    return [[NSURLConnection alloc] initWithRequest:request delegate:delegate];
}

- (NSURLConnection*) register:(NSString *)serverURL phoneNumber:(NSString *)phoneNumber deviceId:(NSString *)deviceId delegate:(id)delegate{
    NSMutableDictionary *params = [[NSMutableDictionary alloc] init];
    [params setObject:phoneNumber   forKey:@"phone_number"];
    [params setObject:@"iOS"        forKey:@"device_type"];
    [params setObject:deviceId      forKey:@"device_id"];
    return [self call:serverURL endpoint:registerEndpoint params:params delegate:delegate];
}

- (NSURLConnection *)retrieve:(id)delegate {
    Settings *settings = [Settings sharedInstance];
    NSMutableDictionary *params = [[NSMutableDictionary alloc] init];
    [params setObject:settings.phone                        forKey:@"phone_number"];
    [params setObject:settings.privateKey                   forKey:@"private_key"];
    return [self call:settings.serverBaseUrl endpoint:retrieveEndpoint params:params delegate:delegate];
}

-(NSURLConnection *)send:(NSString *)phoneNumber recipient:(NSString *)recipient message:(NSString *)message delegate:(id)delegate {
    Settings *settings = [Settings sharedInstance];
    NSMutableDictionary *params = [[NSMutableDictionary alloc] init];
    [params setObject:settings.phone                        forKey:@"phone_number"];
    [params setObject:settings.privateKey                   forKey:@"private_key"];
    [params setObject:recipient                             forKey:@"recipient"];
    [params setObject:message                               forKey:@"message"];
    return [self call:settings.serverBaseUrl endpoint:sendEndpoint params:params delegate:delegate];
}

@end
