//
//  ThreadsTBViewController.m
//  Freem
//
//  Created by mulp on 04/03/13.
//  author: Fabio Cuomo : me@devme.it
//

#import "ThreadsTBViewController.h"

@interface ThreadsTBViewController () {
    DBHandle *dbhandle;
    NSMutableArray *datasource;
}

@end

@implementation ThreadsTBViewController

@synthesize freemThreads;

- (void)viewDidLoad {
    [super viewDidLoad];

    self.navigationItem.hidesBackButton = YES;

    UIBarButtonItem *pickContactButton = [[UIBarButtonItem alloc]
                                          initWithImage:[UIImage imageNamed:@"contacts.png"]
                                          style:UIBarButtonItemStylePlain
                                          target:self action:@selector(pickContactFromAddressBook)];
    
    UIBarButtonItem *pickNumberButton = [[UIBarButtonItem alloc]
                                          initWithImage:[UIImage imageNamed:@"compose.png"]
                                          style:UIBarButtonItemStylePlain
                                          target:self action:@selector(messageToNumber)];

    self.navigationItem.rightBarButtonItems = [NSArray arrayWithObjects:pickNumberButton, pickContactButton, nil];

    UIView *leftView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 150, 32)];
    UIImageView *leftImage = [[UIImageView alloc] initWithFrame:CGRectZero];
    leftImage.image = [UIImage imageNamed:@"freem-icon.png"];
    leftImage.frame = CGRectMake(0, 0, 32, 32);
    [leftView addSubview:leftImage];
    
    UILabel *leftLabel = [[UILabel alloc] initWithFrame:CGRectMake(40, 1, 50, 32)];
    leftLabel.backgroundColor = [UIColor clearColor];
    leftLabel.font = [UIFont boldSystemFontOfSize:16];
    leftLabel.text = @"freem";
    leftLabel.shadowColor = [UIColor colorWithWhite:1 alpha:0.5];
    leftLabel.shadowOffset = CGSizeMake(1,1);
    [leftView addSubview:leftLabel];
    
    self.navigationItem.leftBarButtonItem = [[UIBarButtonItem alloc] initWithCustomView:leftView];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    if ([self isViewLoaded]) {
        [self refreshThreads];
    }
}

//- (void)viewDidAppear:(BOOL)animated {
//    [super viewDidAppear:animated];
//    if ([self isViewLoaded]) {
//        [self refreshThreads];
//    }
//}

-(void) refreshThreads {
    dbhandle = [DBHandle sharedInstance];
    datasource = [dbhandle getThreads];
    [self.tableView reloadData];
}

- (void) messageToNumber {
    MsgToNumberViewController *msgView = [[MsgToNumberViewController alloc] initWithNibName:@"MsgToNumber" bundle:nil];
    msgView.contact = @"";
    [self.navigationController pushViewController:msgView animated:YES];
}
#pragma mark - Address Book Implementation

- (void)displayPerson:(ABRecordRef)person {
    NSString *firstname = (__bridge_transfer NSString*)ABRecordCopyValue(person,
                                                                         kABPersonFirstNameProperty);
    NSString *lastname = (__bridge_transfer NSString*)ABRecordCopyValue(person,
                                                                        kABPersonLastNameProperty);
    NSString *recipient = @"";
    if (lastname!=nil && ![lastname isEqualToString:@""]) {
        recipient = [[firstname stringByAppendingString:@" "] stringByAppendingString:lastname];
    } else {
        recipient = firstname;
    }
    
    NSString* phone = nil;
    ABMultiValueRef phoneNumbers = ABRecordCopyValue(person, kABPersonPhoneProperty);
    if (ABMultiValueGetCount(phoneNumbers) > 0) {
        phone = (__bridge_transfer NSString*)
        ABMultiValueCopyValueAtIndex(phoneNumbers, 0);
        phone = [phone stringByReplacingOccurrencesOfString:@" " withString:@""];
    } else {
        phone = @"";
    }
    CFRelease(phoneNumbers);
    NSLog(@"Recipient = %@ - Phone: %@", recipient, phone);
    
    FreeMThreads *fconv;
    if (self.freemThreads==nil) {
        fconv = [FreeMThreads alloc];
        self.freemThreads = fconv;
    }
    self.freemThreads.idthread = 0;
    self.freemThreads.phoneNumber = phone;
    [self.navigationController pushViewController:self.freemThreads animated:NO];
}

-(void) pickContactFromAddressBook {
    ABPeoplePickerNavigationController *picker = [[ABPeoplePickerNavigationController alloc] init];
    picker.peoplePickerDelegate = self;
    [self presentViewController:picker animated:YES completion:NULL];
}

- (BOOL)peoplePickerNavigationController:(ABPeoplePickerNavigationController *)peoplePicker shouldContinueAfterSelectingPerson:(ABRecordRef)person {
    [self displayPerson:person];
    [self dismissViewControllerAnimated:YES completion:NULL];
    return NO;
}

- (void)peoplePickerNavigationControllerDidCancel:(ABPeoplePickerNavigationController *)peoplePicker {
    [self dismissViewControllerAnimated:YES completion:NULL];
}

- (BOOL)peoplePickerNavigationController:(ABPeoplePickerNavigationController *)peoplePicker shouldContinueAfterSelectingPerson:(ABRecordRef)person property:(ABPropertyID)property identifier:(ABMultiValueIdentifier)identifier {
    return NO;
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    // Return the number of sections.
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    // Return the number of rows in the section.
    return [datasource count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *CellIdentifier = @"Cell";

    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier];
        cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    }
    
    // Componi il counter solo per i messaggi non letti
    UILabel *counter = counter = [[UILabel alloc] initWithFrame:CGRectMake(235.0, 12.0, 30.0, 20.0)];
    Thread *th = (Thread*) [datasource objectAtIndex:indexPath.row];
    if ([th.unread intValue]>0) {
        counter.font = [UIFont boldSystemFontOfSize:14.0];
        counter.textAlignment = NSTextAlignmentCenter;
        counter.textColor = [UIColor whiteColor];
        counter.backgroundColor = [UIColor redColor];
        counter.text = [th.unread stringValue];
    } else {
        counter.textColor = [UIColor whiteColor];
        counter.backgroundColor = [UIColor whiteColor];
        counter.text = @"";
    }
    [cell.contentView addSubview:counter];
    
    // Configure the cell...
    cell.textLabel.text = [Utility retrieveContactNameFromAddressBookByPhone:th.phone];
    cell.detailTextLabel.text = th.phone;
    return cell;
}

#pragma mark -
#pragma mark Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    FreeMThreads *fconv;
    if (self.freemThreads==nil) {
        fconv = [FreeMThreads alloc];
        self.freemThreads = fconv;
    }
    Thread *th = (Thread*) [datasource objectAtIndex:indexPath.row];
    self.freemThreads.idthread = th.idthread;
    self.freemThreads.phoneNumber = th.phone;
    NSLog(@"idthread: %@", th.idthread);
    [self.navigationController pushViewController:self.freemThreads animated:YES];
}

#pragma mark -
#pragma mark Table Push Notification 

- (void)didSaveMessage:(int)idthread phone:(NSString *)phone {
    FreeMThreads *fconv;
    if (self.freemThreads==nil) {
        fconv = [FreeMThreads alloc];
        self.freemThreads = fconv;
    }
    self.freemThreads.idthread = [NSNumber numberWithInt:idthread];
    self.freemThreads.phoneNumber = phone;
    BOOL topMost = ([self.navigationController.topViewController isEqual:self.freemThreads]);
    if (topMost) {
        [self.freemThreads refreshConversation];
        [self.freemThreads scrollToLastMessage];
    } else {
        topMost = ([self.navigationController.topViewController isEqual:self]);
        if (topMost) {
            [self.navigationController pushViewController:self.freemThreads animated:YES];
        } else {
            [self refreshThreads];
        }
    }
}

-(void)incomingMessages:(int)idthread phone:(NSString*)phone {
    BOOL topMost = ([self.navigationController.topViewController isEqual:self.freemThreads]);
    if (topMost) {
        FreeMThreads *fconv;
        if (self.freemThreads==nil) {
            fconv = [FreeMThreads alloc];
            self.freemThreads = fconv;
        }
        self.freemThreads.idthread = [NSNumber numberWithInt:idthread];
        self.freemThreads.phoneNumber = phone;
        [self.freemThreads refreshConversation];
        [self.freemThreads scrollToLastMessage];
    } else {
        [self refreshThreads];
    }
}

@end
